@@ -2,6 +2,7 @@
 from functools import partial
 import time
 
+from shakenfist import constants
 from shakenfist import db
 from shakenfist import etcd
 from shakenfist import exceptions
@@ -16,6 +17,13 @@ class DatabaseBackedObject(object):
     current_version = None
     state_targets = None
 
+    STATE_INITIAL = 'initial'
+    STATE_CREATING = 'creating'
+    STATE_CREATED = 'created'
+    STATE_DELETED = 'deleted'
+    STATE_DELETE_WAIT = 'delete_wait'
+    STATE_ERROR = 'error'
+
     def __init__(self, object_uuid, version=None):
         self.__uuid = object_uuid
         self.__version = version
@@ -44,6 +52,9 @@ class DatabaseBackedObject(object):
     def _db_create(cls, object_uuid, metadata):
         metadata['uuid'] = object_uuid
         etcd.create(cls.object_type, None, object_uuid, metadata)
+        LOG.with_fields({
+            'object_type': cls.object_type
+        }).with_fields(metadata).debug('Object created')
 
     @classmethod
     def _db_get(cls, object_uuid):
@@ -59,14 +70,14 @@ class DatabaseBackedObject(object):
         return o
 
     def _db_get_attribute(self, attribute):
-        retval = etcd.get('attribute/%s' %
-                          self.object_type, self.__uuid, attribute)
+        retval = etcd.get('attribute/%s' % self.object_type,
+                          self.__uuid, attribute)
         if not retval:
             return {}
         return retval
 
     def _db_get_attributes(self, attribute_prefix):
-        for key, data in etcd.get_all('attribute/%s' % type(self).object_type,
+        for key, data in etcd.get_all('attribute/%s' % self.object_type,
                                       self.__uuid, prefix=attribute_prefix):
             yield key, data
 
@@ -74,13 +85,16 @@ class DatabaseBackedObject(object):
         etcd.put('attribute/%s' % self.object_type,
                  self.__uuid, attribute, value)
 
+    def _db_delete_attribute(self, attribute):
+        etcd.delete('attribute/%s' % self.object_type, self.__uuid, attribute)
+
     def get_lock(self, subtype=None, ttl=60, relatedobjects=None, log_ctx=None,
-                 op=None):
+                 op=None, timeout=constants.ETCD_ATTEMPT_TIMEOUT):
         if not log_ctx:
             log_ctx = self.log
         return db.get_lock(self.object_type, subtype, self.uuid, ttl=ttl,
                            relatedobjects=relatedobjects, log_ctx=log_ctx,
-                           op=op)
+                           op=op, timeout=timeout)
 
     def get_lock_attr(self, name, op):
         return db.get_lock('attribute/%s' % self.object_type,
@@ -110,8 +124,7 @@ class DatabaseBackedObject(object):
             if orig.value == new_value:
                 return
 
-            # TODO(mikal): move this to a constant when we have a good set of shared basic states.
-            if orig.value == 'deleted':
+            if orig.value == self.STATE_DELETED:
                 LOG.with_fields(
                     {
                         'uuid': self.uuid,
@@ -176,8 +189,12 @@ def state_filter(states, o):
 # Do not use these filters for instances or nodes, use the more
 # specific ones instead
 active_states_filter = partial(
-    state_filter, ['initial', 'creating', 'created', 'error'])
-inactive_states_filter = partial(state_filter, ['deleted'])
+    state_filter, [DatabaseBackedObject.STATE_INITIAL,
+                   DatabaseBackedObject.STATE_CREATING,
+                   DatabaseBackedObject.STATE_CREATED,
+                   DatabaseBackedObject.STATE_ERROR])
+inactive_states_filter = partial(
+    state_filter, [DatabaseBackedObject.STATE_DELETED])
 
 
 def state_age_filter(delay, o):
