@@ -22,13 +22,16 @@ from shakenfist import instance
 from shakenfist.ipmanager import IPManager
 from shakenfist import logutil
 from shakenfist import networkinterface
-from shakenfist.node import Node
+from shakenfist.node import Node, Nodes, active_states_filter as active_nodes
 from shakenfist.tasks import (
     DeployNetworkTask,
+    HypervisorDestroyNetworkTask,
     UpdateDHCPNetworkTask,
     RemoveDHCPNetworkTask,
     RemoveNATNetworkTask)
-from shakenfist import util
+from shakenfist.util import general as util_general
+from shakenfist.util import network as util_network
+from shakenfist.util import process as util_process
 
 
 LOG, _ = logutil.setup(__name__)
@@ -40,7 +43,8 @@ class Network(dbo):
     state_targets = {
         None: (dbo.STATE_INITIAL, ),
         dbo.STATE_INITIAL: (dbo.STATE_CREATED, dbo.STATE_DELETED, dbo.STATE_ERROR),
-        dbo.STATE_CREATED: (dbo.STATE_DELETED, dbo.STATE_ERROR),
+        dbo.STATE_CREATED: (dbo.STATE_DELETED, dbo.STATE_DELETE_WAIT, dbo.STATE_ERROR),
+        dbo.STATE_DELETE_WAIT: (dbo.STATE_DELETED, dbo.STATE_ERROR),
         dbo.STATE_ERROR: (dbo.STATE_DELETED, dbo.STATE_ERROR),
         dbo.STATE_DELETED: (),
     }
@@ -113,7 +117,6 @@ class Network(dbo):
 
         # Networks should immediately appear on the network node
         db.enqueue('networknode', DeployNetworkTask(uuid))
-        n.add_event('deploy', 'enqueued')
 
         # TODO(andy): Integrate metadata into each object type
         # Initialise metadata
@@ -273,7 +276,7 @@ class Network(dbo):
         if not self.is_created():
             return False
 
-        if self.provide_dhcp and util.is_network_node():
+        if self.provide_dhcp and util_network.is_network_node():
             if not self.is_dnsmasq_running():
                 return False
 
@@ -282,8 +285,13 @@ class Network(dbo):
     def is_created(self):
         """Attempt to ensure network has been created successfully."""
 
+        # The floating network always exists, and would fail the vx_bridge
+        # test we apply to other networks.
+        if self.uuid == 'floating':
+            return True
+
         subst = self.subst_dict()
-        if not util.check_for_interface(subst['vx_bridge'], up=True):
+        if not util_network.check_for_interface(subst['vx_bridge'], up=True):
             self.log.warning('%s is not up', subst['vx_bridge'])
             return False
 
@@ -296,44 +304,59 @@ class Network(dbo):
         callers will wait on a lock before calling this function. In this case
         we definitely need to update the in-memory object model.
         """
-        return self.state.value in (self.STATE_DELETED, self.STATE_ERROR)
+        return self.state.value in (self.STATE_DELETED,
+                                    self.STATE_DELETE_WAIT,
+                                    self.STATE_ERROR)
 
     def _create_common(self):
+        # The floating network does not have a vxlan mesh
+        if self.uuid == 'floating':
+            return
+
         subst = self.subst_dict()
 
-        if not util.check_for_interface(subst['vx_interface']):
-            with util.RecordedOperation('create vxlan interface', self):
-                util.create_interface(
+        if not util_network.check_for_interface(subst['vx_interface']):
+            with util_general.RecordedOperation('create vxlan interface', self):
+                util_network.create_interface(
                     subst['vx_interface'], 'vxlan',
                     'id %(vx_id)s dev %(mesh_interface)s dstport 0'
                     % subst)
-                util.execute(None, 'sysctl -w net.ipv4.conf.'
-                                   '%(vx_interface)s.arp_notify=1' % subst)
+                util_process.execute(None, 'sysctl -w net.ipv4.conf.'
+                                     '%(vx_interface)s.arp_notify=1' % subst)
 
-        if not util.check_for_interface(subst['vx_bridge']):
-            with util.RecordedOperation('create vxlan bridge', self):
-                util.create_interface(subst['vx_bridge'], 'bridge', '')
-                util.execute(None, 'ip link set %(vx_interface)s '
-                                   'master %(vx_bridge)s' % subst)
-                util.execute(None, 'ip link set %(vx_interface)s up' % subst)
-                util.execute(None, 'ip link set %(vx_bridge)s up' % subst)
-                util.execute(None, 'sysctl -w net.ipv4.conf.'
-                                   '%(vx_bridge)s.arp_notify=1' % subst)
-                util.execute(None, 'brctl setfd %(vx_bridge)s 0' % subst)
-                util.execute(None, 'brctl stp %(vx_bridge)s off' % subst)
-                util.execute(None, 'brctl setageing %(vx_bridge)s 0' % subst)
+        if not util_network.check_for_interface(subst['vx_bridge']):
+            with util_general.RecordedOperation('create vxlan bridge', self):
+                util_network.create_interface(subst['vx_bridge'], 'bridge', '')
+                util_process.execute(None, 'ip link set %(vx_interface)s '
+                                     'master %(vx_bridge)s' % subst)
+                util_process.execute(
+                    None, 'ip link set %(vx_interface)s up' % subst)
+                util_process.execute(
+                    None, 'ip link set %(vx_bridge)s up' % subst)
+                util_process.execute(None, 'sysctl -w net.ipv4.conf.'
+                                     '%(vx_bridge)s.arp_notify=1' % subst)
+                util_process.execute(
+                    None, 'brctl setfd %(vx_bridge)s 0' % subst)
+                util_process.execute(
+                    None, 'brctl stp %(vx_bridge)s off' % subst)
+                util_process.execute(
+                    None, 'brctl setageing %(vx_bridge)s 0' % subst)
 
     def create_on_hypervisor(self):
+        # The floating network does not have a vxlan mesh
+        if self.uuid == 'floating':
+            return
+
         with self.get_lock(op='create_on_hypervisor'):
             if self.is_dead():
                 raise DeadNetwork('network=%s' % self)
             self._create_common()
 
-            # TODO(andy): Check with mikal: is this task required here?
-            db.enqueue('networknode', DeployNetworkTask(self.uuid))
-            self.add_event('deploy', 'enqueued')
-
     def create_on_network_node(self):
+        # The floating network does not have a vxlan mesh
+        if self.uuid == 'floating':
+            return
+
         with self.get_lock(op='create_on_network_node'):
             if self.is_dead():
                 raise DeadNetwork('network=%s' % self)
@@ -342,44 +365,45 @@ class Network(dbo):
 
             subst = self.subst_dict()
             if not os.path.exists('/var/run/netns/%(netns)s' % subst):
-                with util.RecordedOperation('create netns', self):
-                    util.execute(None, 'ip netns add %(netns)s' % subst)
+                with util_general.RecordedOperation('create netns', self):
+                    util_process.execute(
+                        None, 'ip netns add %(netns)s' % subst)
 
-            if not util.check_for_interface(subst['vx_veth_outer']):
-                with util.RecordedOperation('create router veth', self):
-                    util.create_interface(
+            if not util_network.check_for_interface(subst['vx_veth_outer']):
+                with util_general.RecordedOperation('create router veth', self):
+                    util_network.create_interface(
                         subst['vx_veth_outer'], 'veth',
                         'peer name %(vx_veth_inner)s' % subst)
-                    util.execute(
+                    util_process.execute(
                         None,
                         'ip link set %(vx_veth_inner)s netns %(netns)s' % subst)
-                    util.execute(
+                    util_process.execute(
                         None,
                         'brctl addif %(vx_bridge)s %(vx_veth_outer)s' % subst)
-                    util.execute(
+                    util_process.execute(
                         None, 'ip link set %(vx_veth_outer)s up' % subst)
-                    util.execute(
+                    util_process.execute(
                         None,
                         '%(in_netns)s ip link set %(vx_veth_inner)s up' % subst)
-                    util.execute(
+                    util_process.execute(
                         None,
                         '%(in_netns)s ip addr add %(router)s/%(netmask)s '
                         'dev %(vx_veth_inner)s' % subst)
 
-            if not util.check_for_interface(subst['egress_veth_outer']):
-                with util.RecordedOperation('create egress veth', self):
-                    util.create_interface(
+            if not util_network.check_for_interface(subst['egress_veth_outer']):
+                with util_general.RecordedOperation('create egress veth', self):
+                    util_network.create_interface(
                         subst['egress_veth_outer'], 'veth',
                         'peer name %(egress_veth_inner)s' % subst)
-                    util.execute(None,
-                                 'brctl addif %(egress_bridge)s '
-                                 '%(egress_veth_outer)s' % subst)
-                    util.execute(None,
-                                 'ip link set %(egress_veth_outer)s up'
-                                 % subst)
-                    util.execute(None,
-                                 'ip link set %(egress_veth_inner)s '
-                                 'netns %(netns)s' % subst)
+                    util_process.execute(None,
+                                         'brctl addif %(egress_bridge)s '
+                                         '%(egress_veth_outer)s' % subst)
+                    util_process.execute(None,
+                                         'ip link set %(egress_veth_outer)s up'
+                                         % subst)
+                    util_process.execute(None,
+                                         'ip link set %(egress_veth_inner)s '
+                                         'netns %(netns)s' % subst)
 
             if self.provide_nat:
                 # We don't always need this lock, but acquiring it here means
@@ -397,68 +421,74 @@ class Network(dbo):
                     subst['floating_gateway'] = self.floating_gateway
                     subst['floating_netmask'] = ipm.netmask
 
-                with util.RecordedOperation('enable virtual routing', self):
-                    addresses = util.get_interface_addresses(
-                        subst['netns'],
-                        subst['egress_veth_inner'])
+                with util_general.RecordedOperation('enable virtual routing', self):
+                    addresses = util_network.get_interface_addresses(
+                        subst['egress_veth_inner'], namespace=subst['netns'])
                     if not subst['floating_gateway'] in list(addresses):
-                        util.execute(None,
-                                     '%(in_netns)s ip addr add '
-                                     '%(floating_gateway)s/%(floating_netmask)s '
-                                     'dev %(egress_veth_inner)s' % subst)
-                        util.execute(None,
-                                     '%(in_netns)s ip link set '
-                                     '%(egress_veth_inner)s up' % subst)
+                        util_process.execute(None,
+                                             '%(in_netns)s ip addr add '
+                                             '%(floating_gateway)s/%(floating_netmask)s '
+                                             'dev %(egress_veth_inner)s' % subst)
+                        util_process.execute(None,
+                                             '%(in_netns)s ip link set '
+                                             '%(egress_veth_inner)s up' % subst)
 
-                    default_routes = util.get_default_routes(subst['netns'])
+                    default_routes = util_network.get_default_routes(
+                        subst['netns'])
                     if default_routes != [subst['floating_router']]:
                         if default_routes:
                             for default_route in default_routes:
-                                util.execute(None,
-                                             '%s route del default gw %s'
-                                             % (subst['in_netns'], default_route))
+                                util_process.execute(None,
+                                                     '%s route del default gw %s'
+                                                     % (subst['in_netns'], default_route))
 
-                        util.execute(None,
-                                     '%(in_netns)s route add default '
-                                     'gw %(floating_router)s' % subst)
+                        util_process.execute(None,
+                                             '%(in_netns)s route add default '
+                                             'gw %(floating_router)s' % subst)
 
                 self.enable_nat()
 
         self.update_dhcp()
+
+        # A final check to ensure we haven't raced with a delete
+        if self.is_dead():
+            raise DeadNetwork('network=%s' % self)
         self.state = self.STATE_CREATED
 
     def delete_on_hypervisor(self):
         with self.get_lock(op='Network delete'):
             subst = self.subst_dict()
 
-            if util.check_for_interface(subst['vx_bridge']):
-                with util.RecordedOperation('delete vxlan bridge', self):
-                    util.execute(None, 'ip link delete %(vx_bridge)s' % subst)
+            if util_network.check_for_interface(subst['vx_bridge']):
+                with util_general.RecordedOperation('delete vxlan bridge', self):
+                    util_process.execute(
+                        None, 'ip link delete %(vx_bridge)s' % subst)
 
-            if util.check_for_interface(subst['vx_interface']):
-                with util.RecordedOperation('delete vxlan interface', self):
-                    util.execute(
+            if util_network.check_for_interface(subst['vx_interface']):
+                with util_general.RecordedOperation('delete vxlan interface', self):
+                    util_process.execute(
                         None, 'ip link delete %(vx_interface)s' % subst)
 
     def delete_on_network_node(self):
         with self.get_lock(op='Network delete'):
             subst = self.subst_dict()
 
-            if util.is_network_node():
-                if util.check_for_interface(subst['vx_veth_outer']):
-                    with util.RecordedOperation('delete router veth', self):
-                        util.execute(
+            if util_network.is_network_node():
+                if util_network.check_for_interface(subst['vx_veth_outer']):
+                    with util_general.RecordedOperation('delete router veth', self):
+                        util_process.execute(
                             None, 'ip link delete %(vx_veth_outer)s' % subst)
 
-                if util.check_for_interface(subst['egress_veth_outer']):
-                    with util.RecordedOperation('delete egress veth', self):
-                        util.execute(
+                if util_network.check_for_interface(subst['egress_veth_outer']):
+                    with util_general.RecordedOperation('delete egress veth', self):
+                        util_process.execute(
                             None,
                             'ip link delete %(egress_veth_outer)s' % subst)
 
                 if os.path.exists('/var/run/netns/%(netns)s' % subst):
-                    with util.RecordedOperation('delete netns', self):
-                        util.execute(None, 'ip netns del %(netns)s' % subst)
+                    with util_general.RecordedOperation('delete netns', self):
+                        util_process.execute(
+                            None, 'ip netns del %(netns)s' % subst)
 
                 if self.floating_gateway:
                     with db.get_lock('ipmanager', None, 'floating', ttl=120,
@@ -470,6 +500,15 @@ class Network(dbo):
 
             self.state = self.STATE_DELETED
 
+        # Ensure that all hypervisors remove this network. This is really
+        # just catching strays, apart from on the network node where we
+        # absolutely need to do this thing.
+        for hyp in Nodes([active_nodes]):
+            db.enqueue(hyp.uuid,
+                       {'tasks': [
+                           HypervisorDestroyNetworkTask(self.uuid)
+                       ]})
+
         self.remove_dhcp()
         self.remove_nat()
 
@@ -498,52 +537,50 @@ class Network(dbo):
         if not self.provide_dhcp:
             return
 
-        if util.is_network_node():
+        if util_network.is_network_node():
             subst = self.subst_dict()
-            with util.RecordedOperation('update dhcp', self):
+            with util_general.RecordedOperation('update dhcp', self):
                 with self.get_lock(op='Network update DHCP'):
                     d = dhcp.DHCP(self, subst['vx_veth_inner'])
                     d.restart_dhcpd()
         else:
             db.enqueue('networknode', UpdateDHCPNetworkTask(self.uuid))
-            self.add_event('update dhcp', 'enqueued')
 
     def remove_dhcp(self):
-        if util.is_network_node():
+        if util_network.is_network_node():
             subst = self.subst_dict()
-            with util.RecordedOperation('remove dhcp', self):
+            with util_general.RecordedOperation('remove dhcp', self):
                 with self.get_lock(op='Network remove DHCP'):
                     d = dhcp.DHCP(self, subst['vx_veth_inner'])
                     d.remove_dhcpd()
         else:
             db.enqueue('networknode', RemoveDHCPNetworkTask(self.uuid))
-            self.add_event('remove dhcp', 'enqueued')
 
     def enable_nat(self):
-        if not util.is_network_node():
+        if not util_network.is_network_node():
             return
 
         subst = self.subst_dict()
-        if not util.nat_rules_for_ipblock(self.network_address):
-            with util.RecordedOperation('enable nat', self):
-                util.execute(None,
-                             'echo 1 > /proc/sys/net/ipv4/ip_forward')
-                util.execute(None,
-                             '%(in_netns)s iptables -A FORWARD '
-                             '-o %(egress_veth_inner)s '
-                             '-i %(vx_veth_inner)s -j ACCEPT' % subst)
-                util.execute(None,
-                             '%(in_netns)s iptables -A FORWARD '
-                             '-i %(egress_veth_inner)s '
-                             '-o %(vx_veth_inner)s -j ACCEPT' % subst)
-                util.execute(None,
-                             '%(in_netns)s iptables -t nat -A POSTROUTING '
-                             '-s %(ipblock)s/%(netmask)s '
-                             '-o %(egress_veth_inner)s '
-                             '-j MASQUERADE' % subst)
+        if not util_network.nat_rules_for_ipblock(self.network_address):
+            with util_general.RecordedOperation('enable nat', self):
+                util_process.execute(None,
+                                     'echo 1 > /proc/sys/net/ipv4/ip_forward')
+                util_process.execute(None,
+                                     '%(in_netns)s iptables -A FORWARD '
+                                     '-o %(egress_veth_inner)s '
+                                     '-i %(vx_veth_inner)s -j ACCEPT' % subst)
+                util_process.execute(None,
+                                     '%(in_netns)s iptables -A FORWARD '
+                                     '-i %(egress_veth_inner)s '
+                                     '-o %(vx_veth_inner)s -j ACCEPT' % subst)
+                util_process.execute(None,
+                                     '%(in_netns)s iptables -t nat -A POSTROUTING '
+                                     '-s %(ipblock)s/%(netmask)s '
+                                     '-o %(egress_veth_inner)s '
+                                     '-j MASQUERADE' % subst)
 
     def remove_nat(self):
-        if util.is_network_node():
+        if util_network.is_network_node():
             if self.floating_gateway:
                 with db.get_lock('ipmanager', None, 'floating', ttl=120,
                                  op='Remove NAT'):
@@ -554,12 +591,15 @@ class Network(dbo):
 
         else:
             db.enqueue('networknode', RemoveNATNetworkTask(self.uuid))
-            self.add_event('remove dhcp', 'enqueued')
 
     def discover_mesh(self):
+        # The floating network does not have a vxlan mesh
+        if self.uuid == 'floating':
+            return
+
         mesh_re = re.compile(r'00:00:00:00:00:00 dst (.*) self permanent')
 
-        stdout, _ = util.execute(
+        stdout, _ = util_process.execute(
             None, 'bridge fdb show brport %(vx_interface)s' % self.subst_dict())
 
         for line in stdout.split('\n'):
@@ -568,6 +608,10 @@ class Network(dbo):
                 yield m.group(1)
 
     def ensure_mesh(self):
+        # The floating network does not have a vxlan mesh
+        if self.uuid == 'floating':
+            return
+
         with self.get_lock(op='Network ensure mesh'):
             # Ensure network was not deleted whilst waiting for the lock.
             if self.is_dead():
@@ -631,17 +675,17 @@ class Network(dbo):
         self.log.info('Adding new mesh element %s', n)
         subst = self.subst_dict()
         subst['node'] = n
-        util.execute(None,
-                     'bridge fdb append to 00:00:00:00:00:00 '
-                     'dst %(node)s dev %(vx_interface)s' % subst)
+        util_process.execute(None,
+                             'bridge fdb append to 00:00:00:00:00:00 '
+                             'dst %(node)s dev %(vx_interface)s' % subst)
 
     def _remove_mesh_element(self, n):
         self.log.info('Removing excess mesh element %s', n)
         subst = self.subst_dict()
         subst['node'] = n
-        util.execute(None,
-                     'bridge fdb del to 00:00:00:00:00:00 dst %(node)s '
-                     'dev %(vx_interface)s' % subst)
+        util_process.execute(None,
+                             'bridge fdb del to 00:00:00:00:00:00 dst %(node)s '
+                             'dev %(vx_interface)s' % subst)
 
     # NOTE(mikal): this call only works on the network node, the API
     # server redirects there.
@@ -654,22 +698,21 @@ class Network(dbo):
             ipaddress.IPv4Address(floating_address))
         subst['inner_address'] = inner_address
 
-        util.execute(None,
-                     'ip link add flt-%(floating_address_as_hex)s-o type veth '
-                     'peer name flt-%(floating_address_as_hex)s-i'
-                     % subst)
-        util.execute(None,
-                     'ip link set flt-%(floating_address_as_hex)s-i netns %(netns)s'
-                     % subst)
-        util.execute(None,
-                     '%(in_netns)s ip addr add %(floating_address)s/32 '
-                     'dev flt-%(floating_address_as_hex)s-i'
-                     % subst)
-        util.execute(None,
-                     '%(in_netns)s iptables -t nat -A PREROUTING '
-                     '-d %(floating_address)s -j DNAT '
-                     '--to-destination %(inner_address)s'
-                     % subst)
+        util_network.create_interface(
+            'flt-%(floating_address_as_hex)s-o' % subst, 'veth',
+            'peer name flt-%(floating_address_as_hex)s-i' % subst)
+        util_process.execute(None,
+                             'ip link set flt-%(floating_address_as_hex)s-i netns %(netns)s'
+                             % subst)
+        util_process.execute(None,
+                             '%(in_netns)s ip addr add %(floating_address)s/32 '
+                             'dev flt-%(floating_address_as_hex)s-i'
+                             % subst)
+        util_process.execute(None,
+                             '%(in_netns)s iptables -t nat -A PREROUTING '
+                             '-d %(floating_address)s -j DNAT '
+                             '--to-destination %(inner_address)s'
+                             % subst)
 
     # NOTE(mikal): this call only works on the network node, the API
     # server redirects there.
@@ -682,10 +725,10 @@ class Network(dbo):
             ipaddress.IPv4Address(floating_address))
         subst['inner_address'] = inner_address
 
-        if util.check_for_interface('flt-%(floating_address_as_hex)s-o' % subst):
-            util.execute(None,
-                         'ip link del flt-%(floating_address_as_hex)s-o'
-                         % subst)
+        if util_network.check_for_interface('flt-%(floating_address_as_hex)s-o' % subst):
+            util_process.execute(None,
+                                 'ip link del flt-%(floating_address_as_hex)s-o'
+                                 % subst)
 
 
 class Networks(dbo_iter):
@@ -693,12 +736,7 @@ class Networks(dbo_iter):
         for _, n in etcd.get_all('network', None):
             if n['uuid'] == 'floating':
                 continue
-
-            n = Network.from_db(n['uuid'])
-            if not n:
-                continue
-
-            out = self.apply_filters(n)
+            out = self.apply_filters(Network(n))
             if out:
                 yield out
 
