@@ -2,27 +2,16 @@ import mock
 
 from shakenfist import exceptions
 from shakenfist.ipmanager import IPManager
-from shakenfist.tests import test_shakenfist
+from shakenfist.tests import base
 
 
-class IPManagerTestCase(test_shakenfist.ShakenFistTestCase):
+class IPManagerTestCase(base.ShakenFistTestCase):
     def test_init(self):
         ipm = IPManager('uuid', '192.168.1.0/24')
         self.assertEqual(
             {
-                '192.168.1.0': True,
-                '192.168.1.255': True
-            }, ipm.in_use)
-
-    def test_init_with_inuse(self):
-        ipm = IPManager('uuid', '192.168.1.0/24',
-                        ['192.168.1.10', '192.168.1.11'])
-        self.assertEqual(
-            {
-                '192.168.1.0': True,
-                '192.168.1.255': True,
-                '192.168.1.10': True,
-                '192.168.1.11': True
+                '192.168.1.0': ('ipmanager', 'uuid'),
+                '192.168.1.255': ('ipmanager', 'uuid')
             }, ipm.in_use)
 
     @mock.patch('shakenfist.db.get_lock')
@@ -31,21 +20,21 @@ class IPManagerTestCase(test_shakenfist.ShakenFistTestCase):
         ipm = IPManager.new('uuid', '192.168.1.0/24')
         self.assertEqual(
             {
-                '192.168.1.0': True,
-                '192.168.1.255': True,
-                '192.168.1.1': True
+                '192.168.1.0': ('ipmanager', 'uuid'),
+                '192.168.1.255': ('ipmanager', 'uuid'),
+                '192.168.1.1': ('ipmanager', 'uuid')
             }, ipm.in_use)
 
     @mock.patch('shakenfist.db.get_ipmanager',
                 return_value={
-                    'ipmanager.v1': {
-                        'in_use': [
-                            '192.168.20.0',
-                            '192.168.20.255',
-                            '192.168.20.1',
-                            '192.168.20.75',
-                            '192.168.20.101'
-                        ],
+                    'ipmanager.v2': {
+                        'in_use': {
+                            '192.168.20.0': ('ipmanager', 'uuid'),
+                            '192.168.20.255': ('ipmanager', 'uuid'),
+                            '192.168.20.1': ('ipmanager', 'uuid'),
+                            '192.168.20.75': ('ipmanager', 'uuid'),
+                            '192.168.20.101': ('ipmanager', 'uuid')
+                        },
                         'ipblock': '192.168.20.0/24'
                     }}
                 )
@@ -53,11 +42,11 @@ class IPManagerTestCase(test_shakenfist.ShakenFistTestCase):
         ipm = IPManager.from_db('floating')
         self.assertEqual(
             {
-                '192.168.20.0': True,
-                '192.168.20.255': True,
-                '192.168.20.1': True,
-                '192.168.20.75': True,
-                '192.168.20.101': True
+                '192.168.20.0': ('ipmanager', 'uuid'),
+                '192.168.20.255': ('ipmanager', 'uuid'),
+                '192.168.20.1': ('ipmanager', 'uuid'),
+                '192.168.20.75': ('ipmanager', 'uuid'),
+                '192.168.20.101': ('ipmanager', 'uuid')
             }, ipm.in_use)
 
     def test_get_address_at_index(self):
@@ -72,34 +61,34 @@ class IPManagerTestCase(test_shakenfist.ShakenFistTestCase):
 
     def test_reserve(self):
         ipm = IPManager('uuid', '192.168.1.0/24')
-        ipm.reserve('192.168.1.10')
+        ipm.reserve('192.168.1.10', ('test', '123'))
         self.assertEqual(
             {
-                '192.168.1.0': True,
-                '192.168.1.255': True,
-                '192.168.1.10': True
+                '192.168.1.0': ('ipmanager', 'uuid'),
+                '192.168.1.255': ('ipmanager', 'uuid'),
+                '192.168.1.10': ('test', '123')
             }, ipm.in_use)
 
     def test_release(self):
         ipm = IPManager('uuid', '192.168.1.0/24')
-        ipm.reserve('192.168.1.10')
+        ipm.reserve('192.168.1.10', ('test', '123'))
         ipm.release('10.0.0.1')
         ipm.release('192.168.1.10')
         self.assertEqual(
             {
-                '192.168.1.0': True,
-                '192.168.1.255': True
+                '192.168.1.0': ('ipmanager', 'uuid'),
+                '192.168.1.255': ('ipmanager', 'uuid')
             }, ipm.in_use)
 
     def test_is_free_and_reserve(self):
         ipm = IPManager('uuid', '192.168.1.0/24')
         self.assertEqual(True, ipm.is_free('192.168.1.24'))
-        ipm.reserve('192.168.1.24')
+        ipm.reserve('192.168.1.24', ('test', '123'))
         self.assertEqual(False, ipm.is_free('192.168.1.24'))
-        self.assertEqual(False, ipm.reserve('192.168.1.24'))
+        self.assertEqual(False, ipm.reserve('192.168.1.24', ('test', '123')))
 
         self.assertEqual(True, ipm.is_free('192.168.1.42'))
-        self.assertEqual(True, ipm.reserve('192.168.1.42'))
+        self.assertEqual(True, ipm.reserve('192.168.1.42', ('test', '123')))
         self.assertEqual(False, ipm.is_free('192.168.1.42'))
         ipm.release('192.168.1.42')
         self.assertEqual(True, ipm.is_free('192.168.1.42'))
@@ -108,7 +97,7 @@ class IPManagerTestCase(test_shakenfist.ShakenFistTestCase):
         ipm = IPManager('uuid', '10.0.0.0/8')
 
         for _ in range(65025):
-            ipm.get_random_free_address()
+            ipm.get_random_free_address(('test', '123'))
 
         # The extra two are the reserved network and broadcast
         # addresses
@@ -119,7 +108,7 @@ class IPManagerTestCase(test_shakenfist.ShakenFistTestCase):
 
         try:
             for _ in range(65025):
-                ipm.get_random_free_address()
+                ipm.get_random_free_address(('test', '123'))
 
         except exceptions.CongestedNetwork:
             pass
